import abc
from typing import Tuple, Union

import numpy as np
from numpy.random import default_rng

from badgers.core.base import GeneratorMixin
from badgers.core.decorators.time_series import preprocess_inputs
from badgers.generators.time_series.utils import generate_random_patterns_indices


class TrendsGenerator(GeneratorMixin):
    """
    Base class for transformers that generate trends in time-series data
    """

    def __init__(self, random_generator=default_rng(seed=0)):
        """
        :param random_generator: a random number generator
        """
        self.random_generator = random_generator

    @abc.abstractmethod
    def generate(self, X, y, **params) -> Tuple:
        pass


class GlobalAdditiveLinearTrendGenerator(TrendsGenerator):
    """
    Add a linear trend to the input time-series data
    """

    def __init__(self, random_generator=default_rng(seed=0)):
        super().__init__(random_generator=random_generator)

    @preprocess_inputs
    def generate(self, X, y, slope) -> Tuple:
        """

        :param X: the input signal to be transformed
        :param y: not changed (here for API compatibility)
        :param slope: the slope of the trend (increase per time unit)
        :type slope: Union[float | list]
        :return: the transformed signal Xt (X + linear trend), and y (not changed)
        """

        offset = np.linspace(0, slope * len(X), len(X))
        Xt = X + offset
        return Xt, y


class AdditiveLinearTrendGenerator(TrendsGenerator):
    """
    Add a linear trend to the input time-series data
    """

    def __init__(self, random_generator=default_rng(seed=0)):
        super().__init__(random_generator=random_generator)

    @preprocess_inputs
    def generate(self, X, y, slope, start: int, end: int) -> Tuple:
        """


        :param X: the input signal to be transformed
        :param y: not changed (here for API compatibility)
        :param slope: (increase per time unit)
        :type slope: Union[float | list]
        :param end:
        :param start:
        :return: the transformed signal Xt (X + linear trend), and y (not changed)
        """
        if start is None:
            # when start is not given, it is chosen randomly in the first half of the signal
            start = self.random_generator.uniform(0, int(0.5 * len(X)))

        if end is None:
            # when end is not given, the trend will last until the end of the signal
            end = len(X)

        # computing offset:
        # - 0s until "start"
        # - from "start" to "end": linear trend with slope "slope",
        # - from "end" on: the last value
        offset = np.zeros(shape=X.shape)
        offset[start:end, :] = np.linspace(0, slope * (end - start), end - start)
        offset[end:, :] = offset[end - 1, :]

        Xt = X + offset
        return Xt, y


class RandomlySpacedLinearTrends(TrendsGenerator):
    """
    Generates randomly time intervals where a linear trend is added to the signal
    Slopes, Tme intervals locations and widths are chosen randomly
    """

    def __init__(self, random_generator=default_rng(seed=0)):
        super().__init__(random_generator=random_generator)

    @preprocess_inputs
    def generate(self, X, y, n_patterns: int = 10, min_width_pattern: int = 5,
                 max_width_patterns: int = 10, slope_min: float = -0.05, slope_max: float = 0.05) -> Tuple:
        """
        Generates randomly time intervals where a linear trend is added to the signal
        Slopes, Tme intervals locations and widths are chosen randomly.

        :param X:
        :param y:
        :param n_patterns: the total number of time intervals where a linear trend is add
        :param min_width_pattern: the minimum with of the time intervals
        :param max_width_patterns: the maximum with of the time intervals
        :param slope_min: the minimum value of the slope (slope is chosen uniformly at random between min_slope and max_slope for each time interval and each column of X)
        :param slope_max: the maximum value of the slope (slope is chosen uniformly at random between min_slope and max_slope for each time interval and each column of X)

        :return:
        """

        # generate patterns indices and values
        self.patterns_indices_ = generate_random_patterns_indices(
            random_generator=self.random_generator,
            n_patterns=n_patterns,
            signal_size=len(X),
            min_width_pattern=min_width_pattern,
            max_width_patterns=max_width_patterns)

        # generate random slopes
        self.slopes_ = self.random_generator.uniform(low=slope_min, high=slope_max, size=(n_patterns, X.shape[1]))

        offset = np.zeros(shape=X.shape)

        for (start, end), slope in zip(self.patterns_indices_, self.slopes_):
            # computing offset:
            # - don't change until "start"
            # - from "start" to "end": add linear trend with slope "slope",
            # - from "end" on: add the last value
            offset[start:end, :] = np.linspace(offset[start, :], offset[start, :] + slope * (end - start), end - start)
            offset[end:, :] = offset[end - 1, :]

        Xt = X + offset
        return Xt, y



import abc
from typing import Tuple

import numpy as np
import pandas as pd
from numpy.random import default_rng

from badgers.core.base import GeneratorMixin
from badgers.core.decorators.time_series import preprocess_inputs
from badgers.generators.time_series.utils import get_patterns_uniform_probability


class TransmissionErrorGenerator(GeneratorMixin):
    """
    Base class for transformers that generate transmission errors

    Transmission errors are errors that includes values dropped, delay, switching values over time...
    """

    def __init__(self, random_generator=default_rng(seed=0)):
        """
        :param random_generator: A random generator
        """
        self.random_generator = random_generator

    @abc.abstractmethod
    def generate(self, X, y, **params) -> Tuple:
        pass


class RandomTimeSwitchGenerator(TransmissionErrorGenerator):
    """
    Switches time randomly (for now uniformly at random).
    A time series x(t) = {x(0), x(1), x(2), ..., x(t), x(t+1), ..., x(n)} is transformed to
                 x'(t) = {x(0), x(1), x(2), ..., x(t+1), x(t), ..., x(n)}
    """

    def __init__(self, random_generator=default_rng(seed=0)):
        """

        :param random_generator:

        """
        super().__init__(random_generator=random_generator)
        self.switch_indices_ = None

    @preprocess_inputs
    def generate(self, X, y, n_switches: int = 10) -> Tuple:
        """
        Switch `n_switches` values between X[i] and X[i+1] where i is chosen uniformly at random in [0,len(X)-1]

        Nothing happens to y

        :param X:
        :param y:
        :param n_switches: number of switches
        :return: Xt, y the transformed time series data and y (the same as input)
        """
        assert n_switches > 0, 'n_switches should be strictly greater than 0'

        self.switch_indices_ = self.random_generator.choice(len(X) - 1, size=n_switches, replace=False)
        for i in self.switch_indices_:
            tmp = X.iloc[i].copy()
            X.iloc[i] = X.iloc[i + 1].copy()
            X.iloc[i + 1] = tmp
        return X, y


class RandomRepeatGenerator(TransmissionErrorGenerator):
    """
    Repeats randomly values

    a time series x(t) = {x(0), x(1), x(2), x(3), x(4), x(5)} would be transformed to
                 x'(t) = {x(0), x(1), x(1), x(2), x(3), x(4), x(4), x(4), x(5)}

    This simulates a delay in transmission where several values are repeated over time
    """

    def __init__(self, random_generator=default_rng(seed=0)):
        """

        :param random_generator:

        """
        super().__init__(random_generator=random_generator)
        self.repeats_ = None  # to store the indices of the repeats (from the original X) and the length of the repeat

    @preprocess_inputs
    def generate(self, X, y, n_repeats: int = 10, min_nb_repeats: int = 1,
                 max_nb_repeats: int = 10) -> Tuple:
        """

        :param X:
        :param y:
        :param n_repeats: number of values that will be repeated
        :param min_nb_repeats: the minimum number of repeats
        :param max_nb_repeats: the maximum number of repeats
        :return: Xt, y the transformed time series data and y (the same as input)
        """
        assert n_repeats > 0, 'n_repeats should be strictly greater than 0'

        # generate indices for repeats
        self.repeats_ = [(i, l) for i, l in zip(
            sorted(self.random_generator.choice(len(X), size=n_repeats, replace=False, shuffle=False)),
            self.random_generator.integers(low=min_nb_repeats, high=max_nb_repeats, size=n_repeats)
        )]

        Xt = X.values

        # generate the repeats and insert them
        offset = 0

        for i, l in self.repeats_:
            repeat = np.repeat(X.iloc[i], l)
            Xt = np.insert(Xt, i + offset, repeat)
            offset += l

        # create a pandas dataframe with same columns as X
        Xt = pd.DataFrame(data=Xt, columns=X.columns)

        return Xt, y


class RandomDropGenerator(TransmissionErrorGenerator):
    """
    Drops randomly values

    a time series x(t) = {x(0), x(1), x(2), x(3), x(4), x(5)} would be transformed to
                 x'(t) = {x(0), x(1), x(3), x(4)}

    This simulates a problem in transmission where several values are randomly dropped
    """

    def __init__(self, random_generator=default_rng(seed=0)):
        """

        :param random_generator:

        """
        super().__init__(random_generator=random_generator)
        self.drops_indices_ = None  # to store the indices of the drops

    @preprocess_inputs
    def generate(self, X, y, n_drops: int = 10) -> Tuple:
        """

        :param X: time series data
        :param y: not used
        :param n_drops: number of values to drop from the time series
        :return: Xt, y the transformed time series data and y (the same as input)
        """
        assert n_drops > 0, 'n_drops should be strictly greater than 0'

        # generate indices for drops
        self.drops_indices_ = self.random_generator.choice(len(X), size=n_drops, replace=False, shuffle=False)

        Xt = X.drop(X.index[self.drops_indices_], axis=0).reset_index(drop=True)

        return Xt, y


class LocalRegionsRandomDropGenerator(TransmissionErrorGenerator):
    """
    Drops randomly values in specific regions of time

    This simulates a problem in transmission where several values are randomly dropped at specific time intervals (time regions)
    """

    def __init__(self, random_generator=default_rng(seed=0)):
        """

        :param random_generator:

        """
        super().__init__(random_generator=random_generator)
        self.drops_indices_ = None  # to store the indices of the drops
        self.drops_probabilities_ = None # to store the probability of dropping values

    @preprocess_inputs
    def generate(self, X, y, n_drops: int = 10, n_regions: int = 5, min_width_regions: int = 5,
                 max_width_regions: int = 10) -> Tuple:
        """

        :param X: time series data
        :param y: not used
        :param n_drops: number of values to drop from the time series
        :param n_regions: number of time regions (or time intervals) where values will be dropped
        :param min_width_regions: minimum width of the time regions (intervals)
        :param max_width_regions: maximum width of the time regions (intervals)
        :return: Xt, y the transformed time series data and y (the same as input)
        """
        assert n_drops > 0, 'n_drops should be strictly greater than 0'

        # generate probability for where to drop points
        self.drops_probabilities_ = get_patterns_uniform_probability(
            signal_size=len(X),
            random_generator=self.random_generator,
            n_patterns=n_regions,
            min_width_pattern=min_width_regions,
            max_width_patterns=max_width_regions
        )

        # generate indices for drops
        self.drops_indices_ = self.random_generator.choice(len(X), size=n_drops, p=self.drops_probabilities_, replace=False, shuffle=False)

        Xt = X.drop(X.index[self.drops_indices_], axis=0).reset_index(drop=True)

        return Xt, y


class LocalRegionsRandomRepeatGenerator(TransmissionErrorGenerator):
    """
    Repeats randomly values x(t) only in certain time regions (time intervals)

    This simulates a delay in transmission where several values are repeated over time (in certain time intervals)
    """

    def __init__(self, random_generator=default_rng(seed=0)):
        """

        :param random_generator:

        """
        super().__init__(random_generator=random_generator)
        self.repeats_ = None  # to store the indices of the repeats (from the original X) and the length of the repeat
        self.repeats_probabilities_ = None  # to store the probability of repeating values

    @preprocess_inputs
    def generate(self, X, y, n_repeats: int = 10, min_nb_repeats: int = 1,
                 max_nb_repeats: int = 10, n_regions: int = 5, min_width_regions: int = 5,
                 max_width_regions: int = 10) -> Tuple:
        """

        :param X:
        :param y:
        :param n_repeats: number of values that will be repeated
        :param min_nb_repeats: the minimum number of repeats
        :param max_nb_repeats: the maximum number of repeats
        :param n_regions: number of time regions (or time intervals) where values will be dropped
        :param min_width_regions: minimum width of the time regions (intervals)
        :param max_width_regions: maximum width of the time regions (intervals)
        :return: Xt, y the transformed time series data and y (the same as input)
        """
        assert n_repeats > 0, 'n_repeats should be strictly greater than 0'

        # generate probability for where to drop points
        self.repeats_probabilities_ = get_patterns_uniform_probability(
            signal_size=len(X),
            random_generator=self.random_generator,
            n_patterns=n_regions,
            min_width_pattern=min_width_regions,
            max_width_patterns=max_width_regions
        )

        # generate indices for repeats
        self.repeats_ = [(i, l) for i, l in zip(
            sorted(self.random_generator.choice(len(X), size=n_repeats, p=self.repeats_probabilities_, replace=False, shuffle=False)),
            self.random_generator.integers(low=min_nb_repeats, high=max_nb_repeats, size=n_repeats)
        )]

        Xt = X.values

        # generate the repeats and insert them
        offset = 0

        for i, l in self.repeats_:
            repeat = np.repeat(X.iloc[i], l)
            Xt = np.insert(Xt, i + offset, repeat)
            offset += l

        # create a pandas dataframe with same columns as X
        Xt = pd.DataFrame(data=Xt, columns=X.columns)

        return Xt, y


from unittest import TestCase

import numpy as np

from badgers.generators.time_series.utils import generate_random_patterns_indices


class TestGenerateRandomPatternsIndices(TestCase):
    def test_no_patterns(self):
        random_generator = np.random.default_rng(0)
        with self.assertRaises(AssertionError):
            generate_random_patterns_indices(
                random_generator=random_generator,
                signal_size=100,
                n_patterns=0,
                min_width_pattern=5,
                max_width_patterns=10
            )

    def test_single_pattern(self):
        random_generator = np.random.default_rng(0)
        signal_size = 100
        n_patterns = 1
        min_width_pattern = 5
        max_width_patterns = 10
        patterns_indices = generate_random_patterns_indices(
            random_generator=random_generator,
            signal_size=signal_size,
            n_patterns=n_patterns,
            min_width_pattern=min_width_pattern,
            max_width_patterns=max_width_patterns
        )

        self.assertEqual(len(patterns_indices), n_patterns)
        for start, end in patterns_indices:
            self.assertTrue(start < end)
            self.assertTrue(end - start >= min_width_pattern)
            self.assertTrue(end - start < max_width_patterns)
            self.assertTrue(0 <= start < signal_size)
            self.assertTrue(0 <= end < signal_size)

    def test_generate_random_patterns_indices(self):
        random_generator = np.random.default_rng(0)
        signal_size = 100
        n_patterns = 5
        min_width_pattern = 5
        max_width_patterns = 10
        patterns_indices = generate_random_patterns_indices(
            random_generator=random_generator,
            signal_size=signal_size,
            n_patterns=n_patterns,
            min_width_pattern=min_width_pattern,
            max_width_patterns=max_width_patterns
        )

        self.assertEqual(len(patterns_indices), n_patterns)
        for start, end in patterns_indices:
            self.assertTrue(start < end)
            self.assertTrue(end - start >= min_width_pattern)
            self.assertTrue(end - start < max_width_patterns)
            self.assertTrue(0 <= start < signal_size)
            self.assertTrue(0 <= end < signal_size)



import unittest

import numpy as np
import pandas as pd
from numpy.random import default_rng
from pandas._testing import assert_frame_equal

from badgers.generators.time_series.trends import GlobalAdditiveLinearTrendGenerator, AdditiveLinearTrendGenerator, \
    RandomlySpacedLinearTrends


class TestGlobalAdditiveLinearTrendsGenerator(unittest.TestCase):
    def setUp(self):
        self.random_generator = default_rng(seed=0)

    def test_global_additive_linear_trend_generator(self):
        # Test the generate method of GlobalAdditiveLinearTrendGenerator
        global_additive_linear_trend_generator = GlobalAdditiveLinearTrendGenerator(
            random_generator=self.random_generator)
        X = pd.DataFrame(data=np.zeros(shape=(10, 4)), columns=[f'col{i}' for i in range(4)])
        y = None

        slope = np.array([1, 2, 3, 4])

        expected_Xt = pd.DataFrame(
            data=np.array([np.linspace(0, len(X) * s, len(X)) for s in slope]).T,
            columns=X.columns, index=X.index
        )

        Xt, _ = global_additive_linear_trend_generator.generate(X, y, slope=slope)

        assert_frame_equal(Xt, expected_Xt)


class TestAdditiveLinearTrendsGenerator(unittest.TestCase):
    def setUp(self):
        self.random_generator = default_rng(seed=0)

    def test_global_additive_linear_trend_generator(self):
        # Test the generate method of AdditiveLinearTrendGenerator
        additive_linear_trend_generator = AdditiveLinearTrendGenerator(
            random_generator=self.random_generator)
        X = pd.DataFrame(data=np.zeros(shape=(10, 4)), columns=[f'col{i}' for i in range(4)])
        y = None
        start = 3
        end = 7

        slope = np.array([0, 0.5, 1, 2])

        expected_Xt = pd.DataFrame(
            data=np.array(
                [
                    [0., 0., 0., 0.],
                    [0., 0., 0., 0.],
                    [0., 0., 0., 0.],
                    [0., 0., 0., 0.],
                    [0., 2. / 3., 4. / 3., 8. / 3.],
                    [0., 4. / 3., 8. / 3., 16. / 3.],
                    [0., 2., 4., 8.],
                    [0., 2., 4., 8.],
                    [0., 2., 4., 8.],
                    [0., 2., 4., 8.]
                ]
            ),
            columns=X.columns, index=X.index
        )

        Xt, _ = additive_linear_trend_generator.generate(X, y, slope=slope, start=start, end=end)

        assert_frame_equal(Xt, expected_Xt)


class TestRandomlySpacedLinearTrends(unittest.TestCase):
    def setUp(self):
        self.random_generator = default_rng(seed=0)

    def test_global_additive_linear_trend_generator(self):
        # Test the generate method of AdditiveLinearTrendGenerator
        randomly_spaced_trend_generator = RandomlySpacedLinearTrends(
            random_generator=self.random_generator)
        X = pd.DataFrame(data=np.zeros(shape=(100, 4)), columns=[f'col{i}' for i in range(4)])
        y = None

        Xt, _ = randomly_spaced_trend_generator.generate(X, y, n_patterns=5, min_width_pattern=5, max_width_patterns=10)

        # assert outside time intervals, constant values
        for i in range(1, len(randomly_spaced_trend_generator.patterns_indices_)):
            s = randomly_spaced_trend_generator.patterns_indices_[i - 1][1]
            e = randomly_spaced_trend_generator.patterns_indices_[i][0]
            self.assertEqual(Xt[s:e].diff().dropna().sum().sum(), 0)


if __name__ == '__main__':
    unittest.main()



import unittest

import pandas as pd
from numpy.random import default_rng

from badgers.generators.time_series.transmission_errors import RandomTimeSwitchGenerator, RandomRepeatGenerator, \
    RandomDropGenerator, LocalRegionsRandomDropGenerator, LocalRegionsRandomRepeatGenerator


class TestRandomTimeSwitchGenerator(unittest.TestCase):

    def test_no_switch(self):
        X = pd.DataFrame([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
        generator = RandomTimeSwitchGenerator(random_generator=default_rng(seed=0))
        with self.assertRaises(AssertionError):
            generator.generate(X.copy(), y=None, n_switches=0)

    def test_single_switch(self):
        X = pd.DataFrame([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
        generator = RandomTimeSwitchGenerator(random_generator=default_rng(seed=0))
        Xt, _ = generator.generate(X.copy(), y=None, n_switches=1)

        # assert same values in both arrays
        self.assertSetEqual(set(X[0]), set(Xt[0]))
        # assert only 2 values differs (n_switches = 1)
        self.assertEqual((X != Xt).sum().values[0], 2)

    def test_single_switch_frame(self):
        X = pd.DataFrame(
            data=[
                [0, 0, 0],
                [1, 1, 1],
                [2, 2, 2],
                [3, 3, 3],
                [4, 4, 4],
                [5, 5, 5],
                [6, 6, 6],
                [7, 7, 7],
                [8, 8, 8],
                [9, 9, 9]
            ]
        )
        generator = RandomTimeSwitchGenerator(random_generator=default_rng(seed=0))
        Xt, _ = generator.generate(X.copy(), y=None, n_switches=1)

        # assert same values in both arrays
        self.assertSetEqual(set(X[0]), set(Xt[0]))
        # assert only 2 values differs (n_switches = 1)
        self.assertEqual((X != Xt).sum().values[0], 2)


class TestRandomRepeatGenerator(unittest.TestCase):

    def test_no_repeat(self):
        X = pd.DataFrame([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
        generator = RandomRepeatGenerator(random_generator=default_rng(seed=0))
        with self.assertRaises(AssertionError):
            generator.generate(X.copy(), y=None, n_repeats=0, min_nb_repeats=2, max_nb_repeats=3)

    def test_two_repeats(self):
        X = pd.DataFrame([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
        generator = RandomRepeatGenerator(random_generator=default_rng(seed=0))
        Xt, _ = generator.generate(X.copy(), y=None, n_repeats=2, min_nb_repeats=2, max_nb_repeats=3)
        self.assertEqual(Xt.shape[0], X.shape[0] + 2 * 2)
        self.assertSetEqual(set(X[0]), set(Xt[0]))


class TestLocalRegionsRandomRepeatGenerator(unittest.TestCase):

    def test_no_repeat(self):
        X = pd.DataFrame([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
        generator = LocalRegionsRandomRepeatGenerator(random_generator=default_rng(seed=0))
        with self.assertRaises(AssertionError):
            generator.generate(X.copy(), y=None, n_repeats=0, min_nb_repeats=2, max_nb_repeats=3)

    def test_single_repeat_single_region(self):
        X = pd.DataFrame([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
        generator = LocalRegionsRandomRepeatGenerator(random_generator=default_rng(seed=0))
        Xt, _ = generator.generate(X.copy(), y=None, n_repeats=1, min_nb_repeats=2, max_nb_repeats=3, n_regions=1,
                                   min_width_regions=3, max_width_regions=7)
        self.assertEqual(Xt.shape[0], X.shape[0] + 2)
        self.assertSetEqual(set(X[0]), set(Xt[0]))

    def test_many_repeats_single_region(self):
        X = pd.DataFrame([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
        generator = LocalRegionsRandomRepeatGenerator(random_generator=default_rng(seed=0))
        Xt, _ = generator.generate(X.copy(), y=None, n_repeats=3, min_nb_repeats=2, max_nb_repeats=3, n_regions=1,
                                   min_width_regions=3, max_width_regions=7)
        self.assertEqual(Xt.shape[0], X.shape[0] + 2 * 3)
        self.assertSetEqual(set(X[0]), set(Xt[0]))

    def test_many_repeats_many_regions(self):
        X = pd.DataFrame([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
        generator = LocalRegionsRandomRepeatGenerator(random_generator=default_rng(seed=0))
        Xt, _ = generator.generate(X.copy(), y=None, n_repeats=4, min_nb_repeats=2, max_nb_repeats=3, n_regions=2,
                                   min_width_regions=3, max_width_regions=5)
        self.assertEqual(Xt.shape[0], X.shape[0] + 2 * 4)
        self.assertSetEqual(set(X[0]), set(Xt[0]))


class TestRandomDrop(unittest.TestCase):

    def test_no_drop(self):
        X = pd.DataFrame([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
        generator = RandomDropGenerator(random_generator=default_rng(0))
        with self.assertRaises(AssertionError):
            generator.generate(X.copy(), y=None, n_drops=0)

    def test_single_drop(self):
        X = pd.DataFrame([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
        generator = RandomDropGenerator(random_generator=default_rng(0))
        Xt, _ = generator.generate(X.copy(), y=None, n_drops=1)
        self.assertEqual(X.shape[0], Xt.shape[0] + 1)


class TestLocalRegionsRandomDropGenerator(unittest.TestCase):

    def test_no_drop(self):
        X = pd.DataFrame([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
        generator = LocalRegionsRandomDropGenerator(random_generator=default_rng(0))
        with self.assertRaises(AssertionError):
            generator.generate(X.copy(), y=None, n_drops=0, n_regions=0)

    def test_single_drop(self):
        X = pd.DataFrame([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
        generator = LocalRegionsRandomDropGenerator(random_generator=default_rng(0))
        Xt, _ = generator.generate(X.copy(), y=None, n_drops=1, n_regions=1, min_width_regions=2, max_width_regions=4)
        self.assertEqual(X.shape[0], Xt.shape[0] + 1)

    def test_many_drops_single_region(self):
        X = pd.DataFrame(range(100))
        generator = LocalRegionsRandomDropGenerator(random_generator=default_rng(0))
        Xt, _ = generator.generate(X.copy(), y=None, n_drops=5, n_regions=1, min_width_regions=10, max_width_regions=20)
        self.assertEqual(X.shape[0], Xt.shape[0] + 5)


if __name__ == '__main__':
    unittest.main()
